package com.yeejoin.amos.fas.business.controller;


import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.fas.business.service.intfc.IWaterResourceService;
import com.yeejoin.amos.fas.business.vo.ReginParams;
import com.yeejoin.amos.fas.core.common.request.CommonPageable;
import com.yeejoin.amos.fas.core.util.CommonResponse;
import com.yeejoin.amos.fas.core.util.CommonResponseUtil;
import com.yeejoin.amos.fas.dao.entity.WaterResource;
import com.yeejoin.amos.fas.dao.entity.WaterResourceEquipment;
import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@RequestMapping(value = "/api/waterresource")
@Api(tags="水资源API")
public class WaterResourceController extends BaseController{


    @Autowired
    IWaterResourceService iWaterResourceService;

    @ApiOperation(httpMethod = "POST",value = "添加水资源", notes = "添加水资源")
    @RequestMapping(value = "", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    public CommonResponse create(@RequestBody WaterResource waterResource)  throws Exception{

        if(waterResource == null
                || StringUtils.isEmpty(waterResource.getType())
                || StringUtils.isEmpty(waterResource.getFloor3d())
//                || StringUtils.isEmpty(waterResource.getPosition3d())
                || StringUtils.isEmpty(waterResource.getCode() ))
            throw new Exception("数据校验失败.");
		ReginParams reginParams =getSelectedOrgInfo();
		String compCode=getOrgCode(reginParams);
        waterResource.setCreateBy("0");
        waterResource.setCreateDate(new Date());
        waterResource.setOrgCode(compCode);
        return CommonResponseUtil.success(iWaterResourceService.save(waterResource));
    }


    @ApiOperation(httpMethod = "PUT",value = "编辑水资源", notes = "编辑水资源")
    @RequestMapping(value = "/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
    public CommonResponse udpate(@PathVariable Long id, @RequestBody WaterResource waterResource) {
        waterResource.setId(id);
        ReginParams reginParams =getSelectedOrgInfo();
		String compCode=getOrgCode(reginParams);
        waterResource.setOrgCode(compCode);
        return CommonResponseUtil.success(iWaterResourceService.save(waterResource));
    }

 // @Authorization(ingore = true)
    @ApiOperation(httpMethod = "GET",value = "查询单个水资源", notes = "查询单个水资源")
    @RequestMapping(value = "/{id}", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse queryPoint(@PathVariable Long id) {

        return CommonResponseUtil.success(iWaterResourceService.queryOne(id));
    }
    

    @ApiOperation(httpMethod = "DELETE",value = "删除水资源", notes = "查询单个水资源")
    @RequestMapping(value = "/{ids}", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
    public CommonResponse delete(@PathVariable String  ids) throws Exception {
        String [] idArray = ids.split(",");
        return CommonResponseUtil.success(iWaterResourceService.delete(idArray));
    }

    @ApiOperation(httpMethod = "GET",value = "分页查询水资源", notes = "分页查询水资源")
    @RequestMapping(value = "/page", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse forPage(
            @ApiParam(value = "名称模糊查询", required = false) @RequestParam(required = false)String  name,
            @ApiParam(value = "编号模糊查询", required = false) @RequestParam(required = false)String  code,
            @ApiParam(value = "编号模糊查询", required = false) @RequestParam(required = false)String  type,
                                    @RequestParam int pageNumber,
                                    @RequestParam  int pageSize
    ) {
        CommonPageable commonPageable = new CommonPageable(pageNumber,pageSize);
        ReginParams reginParams =getSelectedOrgInfo();
		String compCode = getOrgCode(reginParams);
        return CommonResponseUtil.success(iWaterResourceService.queryForPage(compCode,StringUtils.trimToNull(name),StringUtils.trimToNull(code),StringUtils.trimToNull(type),commonPageable));
    }
    
    //@Authorization(ingore = true)
    @ApiOperation(httpMethod = "GET",value = "查询水资源", notes = "分页查询水资源")
    @RequestMapping(value = "/list", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    public CommonResponse list() {
        return CommonResponseUtil.success(iWaterResourceService.queryForList());
    }
    
    
	@ApiOperation(httpMethod = "POST", value = "绑定消防设备", notes = "绑定消防设备")
	@RequestMapping(value = "/fireEquipmenet", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
	public CommonResponse createFireRelation(@RequestBody List<WaterResourceEquipment> waterResourceEquipments)
			throws Exception {

		if (CollectionUtils.isEmpty(waterResourceEquipments))
			throw new Exception("数据校验失败.");
		iWaterResourceService.saveBindFireEquipment(waterResourceEquipments);
		return CommonResponseUtil.success();
	}
	
	@ApiOperation(httpMethod = "DELETE", value = "解除绑定消防设备", notes = "解除绑定消防设备")
	@RequestMapping(value = "/fireEquipmenet", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
	public CommonResponse deleteFireEqRelation(@RequestBody List<WaterResourceEquipment> waterResourceEquipments)
			throws Exception {

		if (CollectionUtils.isEmpty(waterResourceEquipments))
			throw new Exception("数据校验失败.");
		iWaterResourceService.deleteBindFireEquipment(waterResourceEquipments);
		return CommonResponseUtil.success();
	}

	@ApiOperation(httpMethod = "GET", value = "分页查询已经绑定的消防设备列表", notes = "分页查询已经绑定的消防设备列表")
	@RequestMapping(value = "/{waterResourceId}/fireEquipmenet/page", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
	public CommonResponse queryForEuqimentList(@PathVariable Long waterResourceId, @RequestParam int pageNumber,
			@RequestParam int pageSize) {
		CommonPageable commonPageable = new CommonPageable(pageNumber, pageSize);
		return CommonResponseUtil.success(iWaterResourceService.queryBindFireEquemtPage(waterResourceId, commonPageable));
	}

}
